@@ -1,32 +1,18 @@
-import sys
-from ast import AST
 from antlr4.tree import Tree
-from antlr4.Token import CommonToken
-from antlr4.InputStream import InputStream
-from antlr4 import FileStream, CommonTokenStream
 
-from .plsqlLexer import plsqlLexer
-from .plsqlParser import plsqlParser
-from .plsqlVisitor import plsqlVisitor
+import antlr_ast
+from antlr_ast import AstNode, Speaker
+
+from . import plsql_grammar
 
 # AST -------------------------------------------------------------------------
 # TODO: Finish Unary+Binary Expr
 #       sql_script
 
+
 def parse(sql_text, start='sql_script', strict=False):
-    input_stream = InputStream(sql_text)
+    return antlr_ast.parse(plsql_grammar, AstVisitor(), sql_text, start, strict)
 
-    lexer = plsqlLexer(input_stream)
-    token_stream = CommonTokenStream(lexer)
-    parser = plsqlParser(token_stream)
-    ast = AstVisitor()     
-
-    if strict:
-        error_listener = CustomErrorListener()
-        parser.removeErrorListeners()
-        parser.addErrorListener(error_listener)
-
-    return ast.visit(getattr(parser, start)())
 
 import yaml
 def parse_from_yaml(fname):
@@ -36,7 +22,6 @@ def parse_from_yaml(fname):
         out[start] = [parse(cmd, start) for cmd in cmds]
     return out
 
-from antlr_ast import AstNode, Speaker
 
 class Unshaped(AstNode):
     _fields = ['arr']
@@ -45,6 +30,7 @@ class Unshaped(AstNode):
         self.arr = arr
         self._ctx = ctx
 
+
 class Script(AstNode):
     _fields = ['body']
     _priority = 0
@@ -59,6 +45,7 @@ class Script(AstNode):
 
         return cls(ctx, body = body)
 
+
 class SelectStmt(AstNode):
     _fields = ['pref', 'target_list', 'into_clause', 'from_clause', 'where_clause',
                'hierarchical_query_clause', 'group_by_clause', 'having_clause', 'model_clause', 
@@ -79,6 +66,7 @@ class SelectStmt(AstNode):
 
         return query
 
+
 class Union(AstNode):
     _fields = ['left', 'op', 'right', 'order_by_clause']        
 
@@ -87,7 +75,7 @@ class Union(AstNode):
         # hoists up ORDER BY clauses from the right SELECT statement
         # since the final ORDER BY applies to the entire statement (not just subquery)
         union = cls._from_fields(visitor, ctx)
-        if not isinstance(ctx.right, plsqlParser.SubqueryParenContext):
+        if not isinstance(ctx.right, plsql_grammar.Parser.SubqueryParenContext):
             order_by = getattr(union.right, 'order_by_clause', None)
             union.order_by_clause = order_by
             # remove from right SELECT
@@ -95,16 +83,20 @@ class Union(AstNode):
 
         return union
 
+
 class Identifier(AstNode):
     _fields = ['fields']
 
+
 # TODO
 class Star(AstNode):
     _fields = []
 
+
 class AliasExpr(AstNode):
     _fields = ['expr', 'alias']
 
+
 class BinaryExpr(AstNode):
     _fields = ['left', 'op', 'right']
     _rules = ['IsExpr', 'InExpr', 'BetweenExpr', 'LikeExpr', 'RelExpr', 
@@ -130,17 +122,21 @@ class BinaryExpr(AstNode):
             bin_or_unary.right = right
         return bin_or_unary
 
+
 class UnaryExpr(AstNode):
     _fields = ['op', 'unary_expression->expr']
 
+
 class OrderByExpr(AstNode):
     _fields = ['order_by_elements->expr']
     #_rules = ['order_by_clause']
 
+
 class SortBy(AstNode):
     _fields = ['expression->expr', 'direction', 'nulls']
     #_rules = ['order_by_elements']
 
+
 class JoinExpr(AstNode):
     _fields = ['left', 'join_type', 'table_ref->right',
                'join_on_part->cond', 
@@ -155,6 +151,7 @@ class JoinExpr(AstNode):
 
         return join_expr
 
+
 from collections.abc import Sequence
 class Call(AstNode):
     _fields = ['name', 'pref', 'args', 'function_argument_analytic->args', 'concatenation->args', 'over_clause']
@@ -174,7 +171,7 @@ class Call(AstNode):
 
 # PARSE TREE VISITOR ----------------------------------------------------------
 
-class AstVisitor(plsqlVisitor):
+class AstVisitor(plsql_grammar.Visitor):
     def visitChildren(self, node, predicate=None):
         result = self.defaultResult()
         n = node.getChildCount()
@@ -206,7 +203,12 @@ class AstVisitor(plsqlVisitor):
         return aggregate
 
     def visitTerminal(self, ctx):
-        return ctx.getText()
+        """converting case insensitive keywords and identifiers to lowercase"""
+        text = ctx.getText()
+        quotes = ['\'', '"']
+        if not (text[0] in quotes and text[-1] in quotes):
+            text = text.lower()
+        return text
 
     def visitSql_script(self, ctx):
         return Script._from_sql_script(self, ctx)
@@ -275,15 +277,15 @@ class AstVisitor(plsqlVisitor):
     def visitInExpr(self, ctx):
         return BinaryExpr._from_in_expr(self, ctx)
 
-    visitIsExpr =     visitBinaryExpr
+    visitIsExpr =       visitBinaryExpr
     visitBetweenExpr = visitModExpr
-    visitLikeExpr = visitModExpr
-    visitRelExpr =    visitBinaryExpr
-    visitMemberExpr = visitBinaryExpr
-    visitCursorExpr = visitUnaryExpr
-    visitNotExpr =    visitUnaryExpr
-    visitAndExpr =    visitBinaryExpr
-    visitOrExpr =     visitBinaryExpr
+    visitLikeExpr =    visitModExpr
+    visitRelExpr =     visitBinaryExpr
+    visitMemberExpr =  visitBinaryExpr
+    visitCursorExpr =  visitUnaryExpr
+    visitNotExpr =     visitUnaryExpr
+    visitAndExpr =     visitBinaryExpr
+    visitOrExpr =      visitBinaryExpr
 
 
     # simple dropping of tokens -----------------------------------------------
@@ -323,16 +325,6 @@ class AstVisitor(plsqlVisitor):
     def visitJoin_using_part(self, ctx):
         return self.visitChildren(ctx, predicate = lambda n: not isinstance(n, Tree.TerminalNode))
 
-
-    # converting case insensitive keywords to lowercase -----------------------
-
-    def visitRegular_id(self, ctx):
-        # will be a single terminal node
-        return self.visitChildren(ctx).lower()
-
-    visitOver_clause_keyword = visitRegular_id
-    visitWithin_or_over_clause_keyword = visitRegular_id
-
     # removing parentheses ----------------------------------------------------
 
     @staticmethod
@@ -352,32 +344,9 @@ class AstVisitor(plsqlVisitor):
 #        item._bind_to_visitor(AstVisitor)
 
 
-from antlr4.error.ErrorListener import ErrorListener
-from antlr4.error.Errors import RecognitionException
-class AntlrException(Exception):
-    def __init__(self, msg, orig):
-        self.msg, self.orig = msg, orig
-
-class CustomErrorListener(ErrorListener):
-    def syntaxError(self, recognizer, badSymbol, line, col, msg, e):
-        if e is not None:
-            msg = "line {line}: {col} {msg}".format(line=line, col=col, msg=msg)
-            raise AntlrException(msg, e)
-        else:
-            raise AntlrException(msg, None)
-
-    def reportAmbiguity(self, recognizer, dfa, startIndex, stopIndex, exact, ambigAlts, configs):
-        return
-        #raise Exception("TODO")
-
-    def reportAttemptingFullContext(self, recognizer, dfa, startIndex, stopIndex, conflictingAlts, configs):
-        return
-        #raise Exception("TODO")
-
-    def reportContextSensitivity(self, recognizer, dfa, startIndex, stopIndex, prediction, configs):
-        return
-        #raise Exception("TODO")
-
 import pkg_resources
 speaker_cfg = yaml.load(pkg_resources.resource_stream('antlr_plsql', 'speaker.yml'))
 speaker = Speaker(**speaker_cfg)
+
+if __name__ == "__main__":
+    parse('SELECT id FROM artists WHERE id > 100')
